@@ -16,10 +16,7 @@ from pydantic import BaseModel
 import requests
 import copy
 
-try:
-    from . import models
-except:
-    import models
+from . import models
 
 ####
 #
@@ -28,6 +25,9 @@ except:
 #
 ####
 
+#   Debug
+_debug = False
+
 #   默认转发线程数量限制
 _threads_limit = 5
 _sem = threading.Semaphore(_threads_limit)
@@ -45,13 +45,15 @@ _log_clear_size = 0.5
 _log_data = ""
 
 #   日志输出结构调整
-logging.basicConfig(level=logging.INFO, format="[%(asctime)s] %(levelname)s\n  %(message)s",
+logging.basicConfig(level=logging.INFO,
+                    format="[%(asctime)s] %(levelname)s\n  %(message)s",
                     datefmt="%Y-%m-%d %H:%M:%S")
 
 
 def log(level="debug", msg=""):
     """
     #   设置不同级别的log输出
+    #   attention等级和info同等，只是通过不同显示的颜色提醒一下开发者或用户
     参数说明：
     level:str,    #   log等级，levels = debug, info, attention, warning, error, critical
     msg:str,    #   log信息
@@ -60,6 +62,7 @@ def log(level="debug", msg=""):
     global _max_log_size
     global _max_log_size_unit
     global _log_clear_size
+    global _debug
 
     clearLog(_max_log_size, _max_log_size_unit, _log_clear_size)
 
@@ -68,28 +71,42 @@ def log(level="debug", msg=""):
     #   输出带颜色log需要执行一次os.system("")
     os.system("")
 
-    if level == "debug":
-        logging.debug("\033[32m" + msg + "\033[0m")
+    #   日志记录等级方法
+    logging_func = {
+        "debug": logging.debug,
+        "info": logging.info,
+        "attention": logging.info,
+        "warning": logging.warning,
+        "error": logging.error,
+        "critical": logging.critical
+    }
 
-    elif level == "info":
-        logging.info(msg)
+    logging_color = {
+        #   绿色
+        "debug": ("\033[32m", "\033[0m"),
+        #   白色
+        "info": ("", ""),
+        #   蓝色
+        "attention": ("\033[94m", "\033[0m"),
+        #   黄色
+        "warning": ("\033[93m", "\033[0m"),
+        #   红色
+        "error": ("\033[91m", "\033[0m"),
+        #   红色
+        "critical": ("\033[91m", "\033[0m")
+    }
 
-    elif level == "attention":
-        logging.info("\033[94m" + msg + "\033[0m")
-
-    elif level == "warning":
-        logging.warning("\033[93m" + msg + "\033[0m")
-
-    elif level == "error":
-        logging.error("\033[91m" + msg + "\033[0m")
-
-    elif level == "critical":
-        logging.critical("\033[91m" + msg + "\033[0m")
+    logging_func[level](logging_color[level][0] + msg + logging_color[level][1])
 
     #   时间戳
     log_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
 
-    _log_data += f"[{log_time}] {level.upper()}\n  {msg}\n"
+    #   Debug模式下全记录
+    if _debug:
+        _log_data += f"[{log_time}] {level.upper()}\n  {msg}\n"
+    #   非Debug则记录info等级以上日志
+    else:
+        _log_data += f"[{log_time}] {level.upper()}\n  {msg}\n" if level != "debug" else ""
 
 
 def clearLog(size: int = _max_log_size, unit: str = _max_log_size_unit, clear_size: float = _log_clear_size):
@@ -124,7 +141,7 @@ def clearLog(size: int = _max_log_size, unit: str = _max_log_size_unit, clear_si
             return
 
 
-def loadConfig(config_data: dict={}):
+def loadConfig(config_data: dict = {}):
     """
     #   读取配置
     参数说明：
@@ -137,8 +154,10 @@ def loadConfig(config_data: dict={}):
     global _max_log_size_unit
     global _log_clear_size
     global _sem
+    global _debug
 
     try:
+        #   一般在通过web调用时会有config_data，目前千乘尚未支持
         if config_data:
             _max_log_size = config_data.get("log", {}).get("max_log_size", 2)
 
@@ -146,13 +165,21 @@ def loadConfig(config_data: dict={}):
 
             _log_clear_size = config_data.get("log", {}).get("log_clear_size", 0.5)
 
+            #   当检测到需要Debug时，将日志模式调成Debug输出等级
             if config_data.get("log", {}).get("debug", False):
                 logging.basicConfig(level=logging.DEBUG, format="[%(asctime)s] %(levelname)s\n  %(message)s",
                                     datefmt="%Y-%m-%d %H:%M:%S", force=True)
+                _debug = True
+            #   当不需要Debug时，调回去
+            else:
+                logging.basicConfig(level=logging.INFO, format="[%(asctime)s] %(levelname)s\n  %(message)s",
+                                    datefmt="%Y-%m-%d %H:%M:%S", force=True)
+                _debug = False
 
             _threads_limit = config_data.get("thread", {}).get("threads_limit", 5)
             _sem = threading.Semaphore(_threads_limit)
 
+        #   没受到配置信息就用本地配置文件
         else:
             work_path = os.getcwd()
 
@@ -171,6 +198,11 @@ def loadConfig(config_data: dict={}):
             if config_data.getboolean("log", "debug", fallback=False):
                 logging.basicConfig(level=logging.DEBUG, format="[%(asctime)s] %(levelname)s\n  %(message)s",
                                     datefmt="%Y-%m-%d %H:%M:%S", force=True)
+                _debug = True
+            else:
+                logging.basicConfig(level=logging.INFO, format="[%(asctime)s] %(levelname)s\n  %(message)s",
+                                    datefmt="%Y-%m-%d %H:%M:%S", force=True)
+                _debug = False
 
             _threads_limit = config_data.getint("thread", "threads_limit", fallback=5)
             _sem = threading.Semaphore(_threads_limit)
@@ -178,6 +210,9 @@ def loadConfig(config_data: dict={}):
     except Exception as error:
         log("warning", f"SDK配置文件读取失败，将使用默认配置\n  {error}")
 
+        #   Debug
+        _debug = False
+
         #   默认转发线程数量限制
         _threads_limit = 5
         _sem = threading.Semaphore(_threads_limit)
@@ -192,9 +227,9 @@ def loadConfig(config_data: dict={}):
         _log_clear_size = 0.5
 
 
-def loadData(path: str) -> dict:
+def loadTestData(path: str) -> dict:
     """
-    #   读取json文件内的数据
+    #   读取test文件夹下json文件内的数据
     参数说明：
     path:str,   #   json文件路径
 
@@ -313,14 +348,14 @@ def hotUpdate(command: str, data: dict = {}):
 
     if not os.path.exists(update_packs_dir):
         log("info", "未检测到热更新包，跳过更新环节")
-        return
+        return False
 
     else:
         update_packs = os.listdir(update_packs_dir)
 
     if not update_packs:
         log("info", "未检测到热更新包，跳过更新环节")
-        return
+        return False
 
     elif len(update_packs) == 1:
         log("info", "检测到热更新包，执行更新环节")
@@ -332,7 +367,7 @@ def hotUpdate(command: str, data: dict = {}):
 
     else:
         log("info", "未检测到热更新包，跳过更新环节")
-        return
+        return False
 
     try:
 
@@ -347,8 +382,7 @@ def hotUpdate(command: str, data: dict = {}):
 
             log("info", "检查版本信息中......")
             if not checkUpdateVersion(os.path.join(temp_dir, "plugin.spec.yaml")):
-                setLocalCache(data, "restart")
-                restartProgram(command, True)
+                return False
 
             temp_files = os.listdir(temp_dir)
 
@@ -373,23 +407,25 @@ def hotUpdate(command: str, data: dict = {}):
         log("error", "热更新执行失败，失败原因：\n  {}\n  详细信息：\n{}".format(error, error_trace))
         log("attention", "将在5秒后重启服务")
         time.sleep(5)
-        restartProgram(command, True)
+        return False
 
-    setLocalCache(data, "restart")
-    restartProgram(command, True)
+    return True
 
 
 def findLatestUpdate(update_packs_dir: str, update_packs: list):
     """
-    #   寻找最新传入的更新包0
+    #   寻找最新传入的更新包
     参数说明：
     update_packs_dir:str,      #   更新包文件夹
     update_packs:list,          #   多个更新包
 
     返回创建时间最新的文件
     """
-    update_packs.sort(key=lambda fn: os.path.getctime(os.path.join(update_packs_dir, fn))
-    if not os.path.isdir(os.path.join(update_packs_dir, fn)) else 0)
+
+    def getCreateTime(update_pack):
+        return os.path.getctime(os.path.join(update_packs_dir, update_pack))
+
+    update_packs.sort(key=getCreateTime)
 
     return update_packs[-1]
 
@@ -457,51 +493,29 @@ def compareVersion(version_pack: str, version_now: str):
     version_now = version_now.split(".")
 
     if len(version_pack) != 3 or len(version_now) != 3:
-        log("warning", "版本号格式不正确，请检查版本号设置，并重新传入新的更新包")
+        log("error", "版本号格式不正确，请检查版本号设置，并重新传入新的更新包")
         return False
 
     try:
-        if int(version_pack[0]) > int(version_now[0]):
+
+        if int(version_pack[0]) == int(version_now[0]) \
+                and int(version_pack[1]) == int(version_now[1]) \
+                and int(version_pack[2]) > int(version_now[2]):
             return True
 
-        elif int(version_pack[0]) == int(version_now[0]):
+        if int(version_pack[0]) > int(version_now[0]) or (
+                int(version_pack[0]) == int(version_now[0]) and int(version_pack[1]) > int(version_now[1])):
+            log("error", "大版本更新请重新安装插件")
+            return False
 
-            if int(version_pack[1]) > int(version_now[1]):
-                return True
-
-            elif int(version_pack[1]) == int(version_now[1]):
-
-                if int(version_pack[2]) > int(version_now[2]):
-                    return True
     except:
-        log("warning", "版本号格式不正确，请检查版本号设置，并重新传入新的更新包")
+        log("error", "版本号格式不正确，请检查版本号设置，并重新传入新的更新包")
         return False
 
-    log("warning", "更新包版本低于目前插件版本，已放弃更新")
+    log("error", "更新包版本低于目前插件版本，已放弃更新")
     return False
 
 
-def restartProgram(command: str, updated: bool = False):
-    """
-    #   重启程序
-    参数说明：
-    command:str,    #   命令：run, http, test
-    updated:bool,   #   热更新指示器，仅在尝试完成热更新后为True
-    """
-    log("attention", "正在重启程序......")
-
-    #   工作区目录
-    work_path = os.getcwd()
-
-    #   获取python解释器路径
-    python = sys.executable
-    if updated:
-        #   相当于在命令行窗口输入 python {command} updated
-        os.execl(python, "python", os.path.join(work_path, "main.py"), command, "updated")
-    else:
-        os.execl(python, "python", os.path.join(work_path, "main.py"), command)
-
-
 def setLocalCache(data: dict = {}, cache_name: str = ""):
     """
     #   创建本地临时缓存
@@ -509,7 +523,7 @@ def setLocalCache(data: dict = {}, cache_name: str = ""):
     data:dict,      #   需要缓存的数据
     cache_name:str, #   缓存文件的名称
 
-    创建成功时返回True
+    创建失败时抛出异常
     """
     log("info", "缓存数据中")
     #   工作区目录
@@ -522,7 +536,7 @@ def setLocalCache(data: dict = {}, cache_name: str = ""):
         if not os.path.exists(sdkcache_dir_path):
             os.mkdir(sdkcache_dir_path)
         #   缓存数据位于插件根目录下__sdkcache__文件夹内
-        with open(os.path.join(sdkcache_dir_path, "{}.chariot-128.sdkc".format(cache_name)), "wb") as cache_file:
+        with open(os.path.join(sdkcache_dir_path, f"{cache_name}.chariot.sdkc"), "wb") as cache_file:
             cache_file.write(json.dumps(data).encode())
     except Exception as error:
         error_trace = traceback.format_exc()
@@ -546,7 +560,7 @@ def getLocalCache(cache_name: str = "") -> dict:
 
     try:
         #   缓存数据位于插件根目录下__sdkcache__文件夹内
-        with open(os.path.join(sdkcache_dir_path, "{}.chariot-128.sdkc".format(cache_name)), "rb") as cache_file:
+        with open(os.path.join(sdkcache_dir_path, f"{cache_name}.chariot.sdkc"), "rb") as cache_file:
             cache = json.loads(cache_file.read().decode())
     except Exception as error:
         error_trace = traceback.format_exc()
