@@ -1,9 +1,4 @@
-try:
-    from .base import *
-    from .chariot import *
-except Exception as error:
-    from base import *
-    from chariot import *
+from .chariot import *
 
 import sys
 import json
@@ -29,7 +24,6 @@ def client(plugin):
 
     #   获取命令
     log("info", "获取执行参数：\n  {}".format(sys.argv))
-    command = sys.argv[1]
 
     #   加载配置文件
     loadConfig()
@@ -38,52 +32,31 @@ def client(plugin):
     #   该对象在初始化后会存储所有功能的类，方便调用
     plugin_object = plugin()
 
-    #   检查是否已完成更新
-    if len(sys.argv) >= 3:
-        updated = sys.argv[2]
-        if updated == "updated":
-            updated = True
-        else:
-            updated = False
-    else:
-        updated = False
-
-    if command == "run":
-        if not updated:
-            data = getData()
-            if not data:
-                return
-            hotUpdate(command, data)
-        else:
-            clearUpdateFile()
-            data = getLocalCache("restart")
-
+    if sys.argv.count("run"):
+        data = getData()
         log("info", "执行 run 命令")
         run(data, plugin_object)
 
-    elif command == "http":
-        if not updated:
-            hotUpdate(command)
-        else:
-            clearUpdateFile()
+    elif sys.argv.count("http"):
+        #   默认工作进程
+        workers = 4
+
+        try:
+            workers = int(sys.argv[2])
+        except:
+            pass
+
         log("info", "执行 http 命令")
-        http(plugin_object)
 
-    elif command == "test":
-        if not updated:
-            data = getData()
-            if not data:
-                return
-            hotUpdate(command, data)
-        else:
-            clearUpdateFile()
-            data = getLocalCache("restart")
+        http(workers)
 
+    elif sys.argv.count("test"):
+        data = getData()
         log("info", "执行 test 命令")
         test(data, plugin_object)
 
     else:
-        log("error", "未知的命令：{}".format(command))
+        log("error", "未知的命令，输入-h以获取帮助")
         return
 
 
@@ -95,7 +68,7 @@ def getData() -> dict:
     """
     if len(sys.argv) >= 3:
         testfile_path = sys.argv[2]
-        data = loadData(testfile_path)
+        data = loadTestData(testfile_path)
     else:
         data = sys.stdin.read()
         if type(data) != dict and data:
