@@ -1,9 +1,4 @@
-try:
-    from .base import *
-    from .models import *
-except:
-    from base import *
-    from models import *
+from .base import *
 
 import traceback
 import requests
@@ -16,7 +11,7 @@ class Actions(object):
 
         self._output = {
             "version": "v1",
-            "type": "action_event",
+            "type": "action",
             "body": {}
         }
 
@@ -74,15 +69,23 @@ class Actions(object):
             output = self._buildOutput({}, False, error_trace)
             return output
 
-    def _run(self, input_data: dict, connection_data: dict):
+    def _run(self, input_data: dict, connection_data: dict, config_data: dict = None):
         """
         #   运行全流程
         参数说明：
         input_data:dict,    #   入参数据
         conection_data:dict,    #   连接数据
+        config_data:dict,   #   配置数据
         """
         log("info", "插件运行中")
 
+        if config_data:
+            log("info", "检查到有配置信息")
+            loadConfig(config_data)
+        else:
+            log("info", "未传入配置信息，使用默认配置")
+            loadConfig()
+
         #   运行connection
         output = self._test(connection_data)
 
@@ -133,10 +136,7 @@ class Actions(object):
         status:bool,    #   run执行状态，执行成功为True，不成功为False
         error_trace:str,  #   详细的错误信息，用于给开发人员看
         """
-        try:
-            from .base import _log_data
-        except:
-            from base import _log_data
+        from .base import _log_data
 
         output = self._output
 
@@ -160,7 +160,7 @@ class Triggers(object):
 
         self._output = {
             "version": "v1",
-            "type": "trigger_event",
+            "type": "trigger",
             "body": {}
         }
 
@@ -197,7 +197,7 @@ class Triggers(object):
         conection_data:dict,    #   连接数据
         """
 
-        log("info", f"获取连接器数据：\n  {connection_data}")
+        log("debug", f"获取连接器数据：\n  {connection_data}")
 
         try:
             #   校验入参数据
@@ -222,7 +222,7 @@ class Triggers(object):
             output = self._buildOutput({}, False, error_trace)
             return output
 
-    def _run(self, input_data, connection_data, dispatcher_url, cache_url):
+    def _run(self, input_data, connection_data, dispatcher_url, cache_url, config_data: dict = None):
         """
         #   运行全流程
         参数说明：
@@ -230,6 +230,7 @@ class Triggers(object):
         conection_data:dict,    #   连接数据
         dispatcher_url:str,     #   数据转发URL
         cache_url:str,          #   缓存服务URL
+        config_data:dict,   #   配置数据
         """
         log("info", "插件运行中")
 
@@ -237,6 +238,13 @@ class Triggers(object):
 
         self.cache_url = cache_url
 
+        if config_data:
+            log("info", "检查到有配置信息")
+            loadConfig(config_data)
+        else:
+            log("info", "未传入配置信息，使用默认配置")
+            loadConfig()
+
         #   运行connection
         output = self._test(connection_data)
 
@@ -247,7 +255,7 @@ class Triggers(object):
         #   运行run
         try:
 
-            log("info", f"获取功能数据：\n  {input_data}")
+            log("debug", f"获取功能数据：\n  {input_data}")
 
             #   校验入参数据
             log("info", "校验入参数据")
@@ -354,10 +362,7 @@ class Triggers(object):
         error_trace:str,  #   详细的错误信息，用于给开发人员追踪错误
         """
 
-        try:
-            from .base import _log_data
-        except:
-            from base import _log_data
+        from .base import _log_data
 
         output = self._output
         output["body"]["output"] = output_data
@@ -381,7 +386,7 @@ class IndicatorReceivers(Triggers):
 
     def __init__(self):
         super().__init__()
-        self._output["type"] = "indicator_receiver_event"
+        self._output["type"] = "indicator_receiver"
 
 
 class AlarmReceivers(Triggers):
@@ -391,4 +396,14 @@ class AlarmReceivers(Triggers):
 
     def __init__(self):
         super().__init__()
-        self._output["type"] = "alarm_receiver_event"
+        self._output["type"] = "alarm_receiver"
+
+
+class AssetReceivers(Triggers):
+    """
+    #   资产接收器目前实现原理和触发器没有区别
+    """
+
+    def __init__(self):
+        super().__init__()
+        self._output["type"] = "asset_receiver"
