@@ -27,7 +27,7 @@ from .models import {{ connModel }}, {{ inputModel }}, {{ outputModel }}
 
 
 
-class {{ actionsName }}(Actions):
+class {{ className }}(Actions):
 
     def __init__(self):
         #   初始化
@@ -48,7 +48,7 @@ class {{ actionsName }}(Actions):
     def run(self, params={}):
         #   write your code
         #   可调用 popEmpty() 去除载荷中的空参数
-        #   使用 setLocalCache() 设置本地缓存
+        #   使用 setLocalCache()、getLocalCache() 管理本地缓存
         {% if inputKeys %}{% for key in inputKeys %}
         {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
         ...
@@ -63,7 +63,7 @@ from .models import {{ connModel }}, {{ inputModel }}, {{ outputModel }}
 
 
 
-class {{ triggersName }}(Triggers):
+class {{ className }}(Triggers):
 
     def __init__(self):
         #   初始化
@@ -85,7 +85,7 @@ class {{ triggersName }}(Triggers):
         #   write your code
         #   可调用 popEmpty() 去除载荷中的空参数
         #   使用 self.setCache() 设置在线缓存
-        #   使用 setLocalCache() 设置本地缓存
+        #   使用 setLocalCache()、getLocalCache() 管理本地缓存
         #   返回必须使用 self.send({})
         {% if inputKeys %}{% for key in inputKeys %}
         {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
@@ -102,7 +102,7 @@ from .models import {{ connModel }}, {{ inputModel }}, {{ outputModel }}
 
 
 
-class {{ alarm_receiversName }}(AlarmReceivers):
+class {{ className }}(AlarmReceivers):
 
     def __init__(self):
         #   初始化
@@ -125,7 +125,7 @@ class {{ alarm_receiversName }}(AlarmReceivers):
         #   可调用 popEmpty() 去除载荷中的空参数
         #   返回必须使用 self.send({})
         #   使用 self.setCache() 设置在线缓存
-        #   使用 setLocalCache() 设置本地缓存
+        #   使用 setLocalCache()、getLocalCache() 管理本地缓存
         {% if inputKeys %}{% for key in inputKeys %}
         {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
         ...
@@ -142,7 +142,7 @@ from .models import {{ connModel }}, {{ inputModel }}, {{ outputModel }}
 
 
 
-class {{ indicator_receiversName }}(IndicatorReceivers):
+class {{ className }}(IndicatorReceivers):
 
     def __init__(self):
         #   初始化
@@ -164,7 +164,7 @@ class {{ indicator_receiversName }}(IndicatorReceivers):
         #   write your code
         #   可调用 popEmpty() 去除载荷中的空参数
         #   使用 self.setCache() 设置在线缓存
-        #   使用 setLocalCache() 设置本地缓存
+        #   使用 setLocalCache()、getLocalCache() 管理本地缓存
         #   返回必须使用 self.send({})
         {% if inputKeys %}{% for key in inputKeys %}
         {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
@@ -173,6 +173,46 @@ class {{ indicator_receiversName }}(IndicatorReceivers):
 
 """
 
+asset_receivers_template = """
+from SDK.subassembly import AssetReceivers
+from SDK.base import *
+
+from .models import {{ connModel }}, {{ inputModel }},{{ outputModel }}
+
+
+
+class {{ className }}(AssetReceivers):
+
+    def __init__(self):
+        #   初始化
+        super().__init__()
+        self.name = "{{ name }}"
+        self.inputModel = {{ inputModel }}
+        self.outputModel = {{ outputModel }}
+        self.connModel = {{ connModel }}
+
+
+    def connection(self, data={}):
+        #   write your code
+        {% if connectionKeys %}{% for key in connectionKeys %}
+        {{ key }} = data.get("{{ key }}"){% endfor %}{% else %}
+        ...    
+        {% endif %}
+
+    def run(self, params={}):
+        #   write your code
+        #   可调用 popEmpty() 去除载荷中的空参数
+        #   返回必须使用 self.send({})
+        #   使用 self.setCache() 设置在线缓存
+        #   使用 setLocalCache()、getLocalCache() 管理本地缓存
+        {% if inputKeys %}{% for key in inputKeys %}
+        {{ key }} = params.get("{{ key }}"){% endfor %}{% else %}
+        ...
+        {% endif %}
+
+
+"""
+
 
 action_rest_template = """
 from SDK.subassembly import Actions
@@ -342,15 +382,13 @@ class {{ actionsName }}(Actions):
 actions_test_template = """
 {
   "version": "v3",
-  "type": "action_start",
+  "type": "action",
   "body": {
     "action": "{{ name }}",
     "meta": {},
-    "connection": {
-    },
+    "connection": {{ connectionKeys }},
     "dispatcher": null,
-    "input": {
-    },
+    "input": {{ inputKeys }},
     "config": {
       "log": {
         "max_log_size": 2,
@@ -369,18 +407,16 @@ actions_test_template = """
 triggers_test_template = """
 {
   "version": "v3",
-  "type": "trigger_start",
+  "type": "trigger",
   "body": {
     "trigger": "{{ name }}",
     "meta": {},
-    "connection": {
-    },
+    "connection": {{ connectionKeys }},
     "dispatcher": {
       "url": "http://127.0.0.1:10001/transpond",
       "cache_url": ""
     },
-    "input": {
-    },
+    "input": {{ inputKeys }},
     "enable_web": false,
     "config": {
       "log": {
@@ -400,18 +436,16 @@ triggers_test_template = """
 alarm_receivers_test_template = """
 {
   "version": "v3",
-  "type": "alarm_receiver_start",
+  "type": "alarm_receiver",
   "body": {
     "alarm": "{{ name }}",
     "meta": {},
-    "connection": {
-    },
+    "connection": {{ connectionKeys }},
     "dispatcher": {
       "url": "http://127.0.0.1:10001/transpond",
       "cache_url": ""
     },
-    "input": {
-    },
+    "input": {{ inputKeys }},
     "enable_web": false,
     "config": {
       "log": {
@@ -431,18 +465,45 @@ alarm_receivers_test_template = """
 indicator_receivers_test_template = """
 {
   "version": "v3",
-  "type": "indicator_receiver_start",
+  "type": "indicator_receiver",
   "body": {
     "receiver": "{{ name }}",
     "meta": {},
-    "connection": {
-    },
+    "connection": {{ connectionKeys }},
     "dispatcher": {
       "url": "http://127.0.0.1:10001/transpond",
       "cache_url": ""
     },
-    "input": {
+    "input": {{ inputKeys }},
+    "enable_web": false,
+    "config": {
+      "log": {
+        "max_log_size": 2,
+        "max_log_size_unit": "MB",
+        "log_clear_size": 0.5,
+        "debug": false
+      },
+      "thread": {
+        "threads_limit": 5
+      }
+    }
+  }
+}
+"""
+
+asset_receivers_test_template = """
+{
+  "version": "v3",
+  "type": "asset_receiver",
+  "body": {
+    "asset": "{{ name }}",
+    "meta": {},
+    "connection": {{ connectionKeys }},
+    "dispatcher": {
+      "url": "http://127.0.0.1:10001/transpond",
+      "cache_url": ""
     },
+    "input": {{ inputKeys }},
     "enable_web": false,
     "config": {
       "log": {
@@ -462,14 +523,16 @@ indicator_receivers_test_template = """
 main_template = """#!/usr/bin/env python
 
 from SDK.cli import client
-{% if actionClassees %}
+{% if actions %}
 import actions{% endif %}
-{% if triggerClassees %}
+{% if triggers %}
 import triggers{% endif %}
-{% if indicatorReceiverClassees %}
+{% if indicator_receivers %}
 import indicator_receivers{% endif %}
-{% if alarmReceiverClassees %}
+{% if alarm_receivers %}
 import alarm_receivers{% endif %}
+{% if asset_receivers %}
+import asset_receivers{% endif %}
 
 # 整个程序入口
 
@@ -481,24 +544,29 @@ class {{ pluginName }}(object):
         self.actions = {}
         self.triggers = {}
         self.indicator_receivers = {}
-        self.alarm_receivers = {}      
+        self.alarm_receivers = {} 
+        self.asset_receivers = {}
         
-        {% for actionClass in actionClassees %}
+        {% for actionClass in actions %}
         self.add_actions(actions.{{ actionClass }}())
         {% endfor %}
 
-        {% for triggerClass in triggerClassees %}
+        {% for triggerClass in triggers %}
         self.add_triggers(triggers.{{ triggerClass }}())
         {% endfor %}
 
-        {% for indicatorReceiverClasse in indicatorReceiverClassees %}
-        self.add_indicator_receivers(indicator_receivers.{{ indicatorReceiverClasse }}())
+        {% for indicatorReceiverClass in indicator_receivers %}
+        self.add_indicator_receivers(indicator_receivers.{{ indicatorReceiverClass }}())
         {% endfor %}
         
-        {% for alarmReceiverClasse in alarmReceiverClassees %}
-        self.add_alarm_receivers(alarm_receivers.{{ alarmReceiverClasse }}())
+        {% for alarmReceiverClass in alarm_receivers %}
+        self.add_alarm_receivers(alarm_receivers.{{ alarmReceiverClass }}())
         {% endfor %}
-
+        
+        {% for assetReceiversClass in asset_receivers %}
+        self.add_asset_receivers(asset_receivers.{{ assetReceiversClass }}())
+        {% endfor %}
+        
     def add_connection(self, connect):
         self.connection[connect.name] = connect
 
@@ -513,7 +581,9 @@ class {{ pluginName }}(object):
     
     def add_alarm_receivers(self, alarm_receiver):
         self.alarm_receivers[alarm_receiver.name] = alarm_receiver
-
+        
+    def add_asset_receivers(self, asset_receiver):
+        self.asset_receivers[asset_receiver.name] = asset_receiver
 
 def main():
 
@@ -521,14 +591,33 @@ def main():
 
 
 if __name__ == '__main__':
-
     main()
     
 """
 
-init_template = """{% for name, className in init_list %}
+init_template = """
+import sys
+from importlib import reload
+{% if module == "actions" %}
+from .models import *
+reload(models)
+{% endif %}
+{% for name, className in init_list %}
 from .{{ name }} import {{ className }}
-{% endfor %}"""
+{% endfor %}
+
+def modules_dict():
+
+    reload(sys.modules[__name__])
+    {% if module == "actions" %}{% for name, className in init_list %}reload({{ name }})
+    {% endfor %}{% endif %}
+    return {
+        {% for name, className in init_list %}"{{ name }}": {{ className }},
+        {% endfor %}
+    }
+"""
+
+
 
 help_template = """
 # {{ name }}
@@ -536,7 +625,8 @@ help_template = """
 ## About
 {{ name }}
 
-
+## 版本信息
+- {{ version }}
 
 ## Connection
 
@@ -677,10 +767,10 @@ None
 {% endif %}
 
 
-{% if action_name == 'output' %}
+{% if trigger_name == 'output' %}
 #### Output
 
-{% for field_name, field_data in action_data.items() -%}
+{% for field_name, field_data in trigger_data.items() -%}
 
 {% if loop.index == 1 -%}
 |Name|Type|Required|Description|Default|Enum|
@@ -711,6 +801,233 @@ None
 {% endif %}
 
 
+
+## Alarm Receivers
+
+---
+
+{% if alarm_receivers %}
+
+
+{% for alarm_receiver, alarm_receiverData in alarm_receivers.items() %}
+
+### {{ alarm_receiver }}
+
+
+---
+
+{% for alarm_receiver_name,alarm_receiver_data in alarm_receiverData.items() %}
+
+{% if alarm_receiver_name == 'input' %}
+#### Input
+
+{% for field_name, field_data in alarm_receiver_data.items() -%}
+
+{% if loop.index == 1 -%}
+|Name|Type|Required|Description|Default|Enum|
+|----|----|--------|-----------|-------|----|
+{%- endif %}
+|{%- if field_data.title -%}
+{{ field_data.title['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
+{{ field_data.description['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|
+
+
+{%- endfor %}
+
+{% endif %}
+
+
+{% if alarm_receiver_name == 'output' %}
+#### Output
+
+{% for field_name, field_data in alarm_receiver_data.items() -%}
+
+{% if loop.index == 1 -%}
+|Name|Type|Required|Description|Default|Enum|
+|----|----|--------|-----------|-------|----|
+{%- endif %}
+|{%- if field_data.title -%}
+{{ field_data.title['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
+{{ field_data.description['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|
+
+
+{%- endfor %}
+
+
+{% endif %}
+
+
+
+{% endfor %}
+
+{% endfor %}
+
+{% endif %}
+
+
+
+## Indicator Receivers
+
+---
+
+{% if indicator_receivers %}
+
+
+{% for indicator_receiver, indicator_receiverData in indicator_receivers.items() %}
+
+### {{ indicator_receiver }}
+
+---
+
+{% for indicator_receiver_name,indicator_receiver_data in indicator_receiverData.items() %}
+
+{% if indicator_receiver_name == 'input' %}
+#### Input
+
+{% for field_name, field_data in indicator_receiver_data.items() -%}
+
+{% if loop.index == 1 -%}
+|Name|Type|Required|Description|Default|Enum|
+|----|----|--------|-----------|-------|----|
+{%- endif %}
+|{%- if field_data.title -%}
+{{ field_data.title['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
+{{ field_data.description['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|
+
+
+{%- endfor %}
+
+{% endif %}
+
+
+{% if indicator_receiver_name == 'output' %}
+#### Output
+
+{% for field_name, field_data in indicator_receiver_data.items() -%}
+
+{% if loop.index == 1 -%}
+|Name|Type|Required|Description|Default|Enum|
+|----|----|--------|-----------|-------|----|
+{%- endif %}
+|{%- if field_data.title -%}
+{{ field_data.title['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
+{{ field_data.description['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|
+
+
+{%- endfor %}
+
+
+{% endif %}
+
+
+
+{% endfor %}
+
+{% endfor %}
+
+{% endif %}
+
+
+
+## Asset Receivers
+
+---
+
+{% if asset_receivers %}
+
+
+{% for asset_receiver, asset_receiverData in asset_receivers.items() %}
+
+### {{ asset_receiver }}
+
+---
+
+{% for asset_receiver_name,asset_receiver_data in asset_receiverData.items() %}
+
+{% if asset_receiver_name == 'input' %}
+#### Input
+
+{% for field_name, field_data in asset_receiver_data.items() -%}
+
+{% if loop.index == 1 -%}
+|Name|Type|Required|Description|Default|Enum|
+|----|----|--------|-----------|-------|----|
+{%- endif %}
+|{%- if field_data.title -%}
+{{ field_data.title['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
+{{ field_data.description['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|
+
+
+{%- endfor %}
+
+{% endif %}
+
+
+{% if asset_receiver_name == 'output' %}
+#### Output
+
+{% for field_name, field_data in asset_receiver_data.items() -%}
+
+{% if loop.index == 1 -%}
+|Name|Type|Required|Description|Default|Enum|
+|----|----|--------|-----------|-------|----|
+{%- endif %}
+|{%- if field_data.title -%}
+{{ field_data.title['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.type }}|{{ field_data.required }}|{%- if field_data.description -%}
+{{ field_data.description['zh-CN'] }}
+{%- else -%}
+None
+{%- endif -%}|{{ field_data.default|default('None') }}|{{ field_data.enum|default('None') }}|
+
+
+{%- endfor %}
+
+
+{% endif %}
+
+
+
+{% endfor %}
+
+{% endfor %}
+
+{% endif %}
+
+
+
 ## Types
 
 {% if types %}
@@ -740,12 +1057,6 @@ None
 {% endfor %}
 
 {% endif %}
-
-
-## 版本信息
-- {{ version }}
-
-## 参考引用
 """
 
 indicator_receivers_model_types = """
@@ -876,6 +1187,39 @@ class {{ className }}(BaseModel):
     
 """
 
+asset_receivers_model_types = """
+class Asset(BaseModel):
+    servicer: str = ""
+    device_type: str = ""
+    device_id: str = ""
+    name: str = ""
+    public_ipv4: str = ""
+    inner_ipv4: str = ""
+    location: str = ""
+    department: str = ""
+    description: str = ""
+    mac: str = ""
+    model: str = ""
+    os: str = ""
+    cpu: str = ""
+    ram: str = ""
+    kernel_version: str = ""
+    port: str = ""
+    disk: str = ""
+    software: str = ""
+    tag: Optional[List[str]] = []
+    raw: str = ""
+    created_at: str
+    updated_at: str
+"""
+
+asset_receivers_model_template = """
+class {{ className }}(BaseModel):
+
+    asset: Asset
+    
+"""
+
 testserver_head_template = r"""
 from fastapi import FastAPI,HTTPException
 import uvicorn
