@@ -6,6 +6,8 @@ import jinja2
 import json
 from chariotCore.templates import *
 from copy import deepcopy
+import configparser
+import tarfile
 
 ####
 #
@@ -31,7 +33,8 @@ module_dict = {
     "actions": "动作",
     "triggers": "触发器",
     "indicator_receivers": "情报接收器",
-    "alarm_receivers": "告警接收器"
+    "alarm_receivers": "告警接收器",
+    "asset_receivers": "资产接收器"
 }
 
 logging.basicConfig(level=logging.DEBUG, format="")
@@ -61,15 +64,21 @@ def log(level="debug", msg=""):
 
     elif level == "warning":
         logging.warning(
-            "\033[93m\n====================== 警 告 ======================\n\n" + f"{level.upper()} | {msg}" + "\n\n===================================================\n\033[0m")
+            "\033[93m\n====================== 警 告 ======================\n\n" +
+            f"{level.upper()} | {msg}" +
+            "\n\n===================================================\n\033[0m")
 
     elif level == "error":
         logging.error(
-            "\033[91m\n====================== 错 误 ======================\n\n" + f"{level.upper()} | {msg}" + "\n\n===================================================\n\033[0m")
+            "\033[91m\n====================== 错 误 ======================\n\n" +
+            f"{level.upper()} | {msg}" +
+            "\n\n===================================================\n\033[0m")
 
     elif level == "critical":
         logging.critical(
-            "\033[91m\n=================== 严 重 错 误 ===================\n\n" + f"{level.upper()} | {msg}" + "\n\n===================================================\n\033[0m")
+            "\033[91m\n=================== 严 重 错 误 ===================\n\n" +
+            f"{level.upper()} | {msg}" +
+            "\n\n===================================================\n\033[0m")
 
 
 def pathCheck(path: str) -> str:
@@ -196,10 +205,10 @@ def DataCheck(data: dict):
 
     name = data.get("name")
     if not name.islower():
-        raise Exception(f"插件名称（name）应为全小写：\n{name}")
+        raise Exception(f"插件ID（name）应为全小写：\n{name}")
 
     if name.find(" ") != -1:
-        raise Exception(f'插件名称（name）不能带有空格（" "）\n请用下划线（"_"）代替')
+        raise Exception(f'插件ID（name）不能带有空格（" "）\n请用下划线（"_"）代替')
 
 
 def generateBaseFile(work_path: str):
@@ -313,11 +322,61 @@ def generateConnectionModel(connection_params: dict) -> str:
     return connection_model
 
 
-def generateModel(module: str, module_data: dict) -> str:
+def generateModule(data, module, types_model, connection_model, connection_keys):
+    """
+    #   生成组件
+    参数说明：
+    data:dict,  #   插件定义数据
+    module:str, #   组件
+    types_model:str,    #   自定义类型校验数据
+    connection_model:str,   #   连接器校验数据
+    connection_keys:list of str,    #   连接器参数列表
+    """
+    module_data = data.get(module)
+
+    if not module_data:
+        log("attention", f"未检测到{module_dict[module]}，跳过{module_dict[module]}的生成")
+        return []
+
+    #   生成校验数据和校验文件
+    generateModel(module, module_data, types_model, connection_model)
+
+    modules_list = generateModuleFile(os.getcwd(), module, module_data, connection_keys)
+
+    return modules_list
+
+
+def generateModel(module, module_data, types_model, connection_model):
+    """
+    #   生成校验数据和校验文件
+    参数说明：
+    module:str, #   组件
+    module_data:str,    #   组件数据
+    types_model:str,    #   自定义类型校验数据
+    connection_model:str,   #   连接器校验数据
+    """
+
+    module_model = generateModelData(module, module_data)
+
+    models_dict = {
+        "actions": model_header + types_model + connection_model + module_model,
+        "triggers": model_header + types_model + connection_model + module_model,
+        "alarm_receivers": model_header + alarm_receivers_model_types +
+                           types_model + connection_model + module_model,
+        "indicator_receivers": model_header + indicator_receivers_model_types +
+                               types_model + connection_model + module_model,
+        "asset_receivers": model_header + asset_receivers_model_types +
+                           types_model + connection_model + module_model
+    }
+
+    generateModelFile(os.getcwd(), module, models_dict[module])
+
+
+def generateModelData(module: str, module_data: dict) -> str:
     """
     #   根据插件定义文件中各个组件的参数生成组件的入参和出参校验数据
     参数说明：
-    module:str,     #   组件，module = actions, triggers, indicator_receivers, alarm_receivers
+    module:str,     #   组件，module = actions, triggers, indicator_receivers, alarm_receivers, asset_receivers
     module_data:dict,    #   插件定义文件中组件的参数
 
     返回需要在models.py文件中写入的动作的入参和出参校验数据
@@ -328,7 +387,6 @@ def generateModel(module: str, module_data: dict) -> str:
     model = ""
 
     for module_key, key_params in module_data.items():
-
         log("info", f"生成 {module_key} 校验数据中")
 
         #   获取输入与输出载荷
@@ -357,13 +415,14 @@ def generateModel(module: str, module_data: dict) -> str:
 
         model += renderStrTemplate(input_data, model_template)
 
-        #   告警接收器和情报接收器的出参校验数据在转发信息时使用，所以校验数据特殊
-        if module == "alarm_receivers":
-            model += renderStrTemplate(output_data, alarm_receivers_model_template)
-        elif module == "indicator_receivers":
-            model += renderStrTemplate(output_data, indicator_receivers_model_template)
-        else:
-            model += renderStrTemplate(output_data, model_template)
+        #   告警接收器、情报接收器和资产接收器的出参校验数据在转发信息时使用，所以校验数据特殊
+        model_template_dict = {
+            "alarm_receivers": alarm_receivers_model_template,
+            "indicator_receivers": indicator_receivers_model_template,
+            "asset_receivers": asset_receivers_model_template
+        }
+
+        model += renderStrTemplate(output_data, model_template_dict.get(module, model_template))
 
         log("info", f"{module_key} 校验数据生成完成")
 
@@ -374,7 +433,7 @@ def generateModel(module: str, module_data: dict) -> str:
 
 def generateModelFile(work_path: str, model_module: str, model_data: str):
     """
-    #   根据目录及校验数据生成校验文件model.py
+    #   根据目录及校验数据生成校验文件models.py
     参数说明：
     work_path:str,    #   当前工作区绝对路径
     model_module:str,     #   校验的组件，model_module = actions, triggers, indicator_receivers, alarm_receivers
@@ -405,12 +464,12 @@ def generateModelFile(work_path: str, model_module: str, model_data: str):
 
 def generateModuleFile(work_path: str, module: str, module_data: dict, connection_keys: list) -> list:
     """
-    #   生成所有功能的运行文件
+    #   生成所有组件的文件
     参数说明：
     work_path:str,    #   当前工作区绝对路径
     module:str,     #   组件，module = actions, triggers, indicator_receivers, alarm_receivers
     module_data:dict,    #   组件的参数
-    connection_keys:list,   #   连接器参数列表
+    connection_keys:list of str,    #   连接器参数列表
 
     返回功能列表 key_list，用于生成入口文件 main.py
     文件生成失败时抛出异常
@@ -419,128 +478,123 @@ def generateModuleFile(work_path: str, module: str, module_data: dict, connectio
     #   用于记录要在初始化文件（__init__.py）中写入的功能导入（import）
     init_list = []
 
-    #   功能列表 key_list，用于生成入口文件 main.py
-    key_list = []
-
     #   组件文件夹路径
-    key_path = os.path.join(work_path, module)
+    module_path = os.path.join(work_path, module)
 
     log("info", f"生成 {module_dict[module]}（{module}）所有功能中")
 
     #   遍历组件内每一个功能和功能的参数
-    for module_key, params in module_data.items():
+    for func_id, params in module_data.items():
         try:
 
-            log("info", rf"生成 {module}\{module_key}.py 文件中")
+            log("info", rf"生成 {module}\{func_id}.py 文件中")
 
-            #   存储用于渲染功能文件内容的数据
-            key_data = {}
+            #   生成功能数据
+            func, func_data, init = generateModuleData(func_id, params, module, connection_keys)
 
-            #   根据不同组件使用不同的文件生成模板
-            if module == "actions":
-                #   功能类名采用全大写
-                key_class_name = module_key.upper() + "_ACTION"
-                key_data["actionsName"] = key_class_name
+            init_list.append(init)
 
-                init_list.append([module_key, key_class_name])
-
-                if params.get("input"):
-                    input_keys = list(params["input"].keys())
-                else:
-                    input_keys = []
-
-                key_template = action_template
-                key_test_template = actions_test_template
-
-            elif module == "triggers":
-
-                key_class_name = module_key.upper() + "_TRIGGER"
-                key_data["triggersName"] = key_class_name
-
-                init_list.append([module_key, key_class_name])
-
-                if params.get("input"):
-                    input_keys = list(params["input"].keys())
-                else:
-                    input_keys = []
-
-                key_template = triggers_template
-                key_test_template = triggers_test_template
-
-            elif module == "indicator_receivers":
-
-                key_class_name = module_key.upper() + "_INDICATOR_RECEIVER"
-                key_data["indicator_receiversName"] = key_class_name
-
-                init_list.append([module_key, key_class_name])
-
-                if params.get("input"):
-                    input_keys = list(params["input"].keys())
-                else:
-                    input_keys = []
-
-                key_template = indicator_receivers_template
-                key_test_template = indicator_receivers_test_template
-
-            elif module == "alarm_receivers":
-
-                key_class_name = module_key.upper() + "_ALARM_RECEIVER"
-                key_data["alarm_receiversName"] = key_class_name
-
-                init_list.append([module_key, key_class_name])
-
-                if params.get("input"):
-                    input_keys = list(params["input"].keys())
-                else:
-                    input_keys = []
-
-                key_template = alarm_receivers_template
-                key_test_template = alarm_receivers_test_template
-
-            else:
-                raise Exception(f"未知的模块：{module}")
-
-            #   功能列表，用于生成入口文件 main.py
-            key_list.append(key_class_name)
-
-            #   设置input, output类名
-            input_class_name = module_key.upper() + "_INPUT"
-            output_class_name = module_key.upper() + "_OUTPUT"
-
-            key_data.update({
-                "name": module_key,
-                "inputModel": input_class_name,
-                "outputModel": output_class_name,
-                "connModel": "CONNECTION",
-                "connectionKeys": connection_keys,
-                "inputKeys": input_keys
-            })
-
-            #   渲染组件内不同功能文件内容
-            key = renderStrTemplate(key_data, key_template)
-
-            file_path = os.path.join(key_path, f"{module_key}.py")
-
-            #   当功能文件存在时不写入，以防覆盖原本已写的功能
-            if not os.path.exists(file_path):
-                with open(file_path, "w", encoding="utf-8", newline="\n") as file:
-                    file.write(key)
-            else:
-                log("attention", rf"{module}\{module_key}.py 已存在，已跳过生成")
-
-            log("info", rf"{module}\{module_key}.py 生成完成")
+            #   生成功能文件
+            generateFuncFile(module, func_id, func)
 
         except Exception as error:
-            raise Exception(rf"{module}\{module_key}.py 生成失败，原因未知：" + f"\n{error}")
+            raise Exception(rf"{module}\{func_id}.py 生成失败，原因未知：" + f"\n{error}")
+
+        module_test_template_dict = {
+            "actions": actions_test_template,
+            "triggers": triggers_test_template,
+            "alarm_receivers": alarm_receivers_test_template,
+            "indicator_receivers": indicator_receivers_test_template,
+            "asset_receivers": asset_receivers_test_template
+        }
 
         #   生成测试用的json文件
-        generateTestFile(work_path, key_data, key_test_template)
+        generateTestFile(func_data, module_test_template_dict[module])
 
-    generateInitFile(module, key_path, init_list)
+    #   生成插件功能初始化文件
+    generateInitFile(module, module_path, init_list)
 
     log("info", f"{module_dict[module]}（{module}）所有功能生成完成")
 
-    return key_list
+    #   功能类名称列表 func_class_name_list，用于生成入口文件 main.py
+    func_class_name_list = [func[1] for func in init_list]
+
+    return func_class_name_list
+
+
+def generateFuncFile(module, func_id, func):
+    """
+    #   生成功能文件
+    module:str,     #   组件，module = actions, triggers, indicator_receivers, alarm_receivers, asset_receivers
+    func_id:str,    #   功能的id
+    func:str,       #   功能的数据
+    """
+
+    module_path = os.path.join(os.getcwd(), module)
+
+    func_path = os.path.join(module_path, f"{func_id}.py")
+
+    if os.path.exists(func_path):
+        log("attention", rf"{module}\{func_id}.py 已存在，已跳过生成")
+    else:
+        with open(func_path, "w", encoding="utf-8", newline="\n") as file:
+            file.write(func)
+        log("info", rf"{module}\{func_id}.py 生成完成")
+
+
+def generateModuleData(func_id, params, module, connection_keys) -> (str, dict, list):
+    """
+    #   生成组件内功能数据
+    参数说明：
+    func_id:str,   #   功能的id
+    params:dict,    #   功能的参数，包括title、input等，重点是input
+    module:str,     #   组件，module = actions, triggers, indicator_receivers, alarm_receivers, asset_receivers
+    connection_keys:list of str,    #   连接器参数列表
+    """
+
+    init = None
+
+    class_name_dict = {
+        "actions": "_ACTION",
+        "triggers": "_TRIGGER",
+        "alarm_receivers": "_ALARM_RECEIVER",
+        "indicator_receivers": "_INDICATOR_RECEIVER",
+        "asset_receivers": "_ASSET_RECEIVER"
+    }
+
+    module_template_dict = {
+        "actions": action_template,
+        "triggers": triggers_template,
+        "alarm_receivers": alarm_receivers_template,
+        "indicator_receivers": indicator_receivers_template,
+        "asset_receivers": asset_receivers_template
+    }
+
+    class_name = func_id.upper() + class_name_dict[module]
+
+    #   用于记录要在初始化文件（__init__.py）中写入的功能导入（import）
+    #   同时用于生成入口文件 main.py
+    init = (func_id, class_name)
+
+    #   这里收集input的key是为了自动生成获取数据的功能
+    if params.get("input"):
+        input_keys = list(params["input"].keys())
+    else:
+        input_keys = []
+
+    func_data = {
+        "name": func_id,
+        "inputModel": func_id.upper() + "_INPUT",
+        "outputModel": func_id.upper() + "_OUTPUT",
+        "connModel": "CONNECTION",
+        "connectionKeys": connection_keys,
+        "inputKeys": input_keys,
+        "className": class_name
+    }
+
+    func = renderStrTemplate(func_data, module_template_dict[module])
+
+    return func, func_data, init
 
 
 def autoGenerateDataTrans(data: dict) -> dict:
@@ -750,7 +804,7 @@ def autoGenerateActionsFile(work_path: str, module_data: dict) -> list:
             raise Exception(rf"actions\{module_key}.py 生成失败，原因未知：" + f"\n{error}")
 
         #   生成测试用的json文件
-        generateTestFile(work_path, key_data, key_test_template)
+        generateTestFile(key_data, key_test_template)
 
     generateInitFile("actions", key_path, init_list)
 
@@ -759,17 +813,16 @@ def autoGenerateActionsFile(work_path: str, module_data: dict) -> list:
     return key_list
 
 
-def generateTestFile(work_path, key_data, key_test_template):
+def generateTestFile(func_data, func_test_template):
     """
     #   生成测试用的json文件
     参数说明：
-    work_path:str,    #   当前工作区绝对路径
-    key_data:dict,     #   功能的各项参数
+    func_data:dict,     #   功能的各项参数
     key_test_template:str,  #   需要渲染的测试文件模板
     生成失败时抛出异常
     """
 
-    module_key = key_data.get("name")
+    module_key = func_data.get("name")
 
     if not module_key:
         raise Exception("功能名称缺少")
@@ -777,10 +830,23 @@ def generateTestFile(work_path, key_data, key_test_template):
     log("info", rf"生成 tests\{module_key}.json 测试文件中")
 
     try:
-        #   渲染测试文件内容
-        test = renderStrTemplate(key_data, key_test_template)
 
-        tests_path = os.path.join(work_path, "tests")
+        connection_params_dict = {}
+        for key in func_data.get("connectionKeys", []):
+            connection_params_dict[key] = None
+
+        func_data["connectionKeys"] = json.dumps(connection_params_dict)
+
+        input_params_dict = {}
+        for key in func_data.get("inputKeys", []):
+            input_params_dict[key] = None
+
+        func_data["inputKeys"] = json.dumps(input_params_dict)
+
+        #   渲染测试文件内容
+        test = renderStrTemplate(func_data, func_test_template)
+
+        tests_path = os.path.join(os.getcwd(), "tests")
         file_path = os.path.join(tests_path, f"{module_key}.json")
         #   当测试文件存在时不写入，以防覆盖原本的测试数据
         if not os.path.exists(file_path):
@@ -806,7 +872,11 @@ def generateInitFile(module, key_path, init_list):
     try:
         log("info", rf"生成 初始化文件 {module}\__init__.py 中")
         file_path = os.path.join(key_path, "__init__.py")
-        init = renderStrTemplate({"init_list": init_list}, init_template)
+        template_data = {
+            "init_list": init_list,
+            "module": module
+        }
+        init = renderStrTemplate(template_data, init_template)
         with open(file_path, "w", encoding="utf-8", newline="\n") as file:
             file.write(init)
         log("info", rf"初始化文件 {module}\__init__.py 生成完成")
@@ -819,6 +889,7 @@ def updateModuleFile(work_path: str):
     #   对1.2.6版本sdk做的actions, triggers, alarm_receivers, indicator_receivers进行兼容性改动，主要改动为规范化类名
     #   对1.2.7版本所使用的去除空参数功能进行修正
     #   对旧requirements.txt添加python-multipart库
+    #   对1.3.0版本之前的Dockerfile进行修正
 
     参数说明：
     work_path:str,    #   当前工作区绝对路径
@@ -884,24 +955,27 @@ def updateModuleFile(work_path: str):
                 with open(requirements_path, "a", encoding="utf-8") as f:
                     f.write("\npython-multipart\n")
 
+        config_path = os.path.join(work_path, "config.ini")
+        if os.path.exists(config_path):
+            with open(config_path, "r", encoding="utf-8") as f:
+                data = f.read()
+            if "[ignore]" not in data:
+                data += "\n[ignore]\n#   打包时忽略的根目录下的文件\n#   以逗号分隔多个文件（文件夹）\nignore = env,venv,tests,.git,.vs,.gitignore,.idea,__sdkcache__,__pycache__,temp_image.tar.gz,temp_plugin.tar.gz"
+                with open(config_path, "w", encoding="utf-8") as f:
+                    f.write(data)
+
     except Exception as error:
         raise Exception(f"兼容措施执行失败，错误未知：\n{error}")
 
 
 def generateMainFile(work_path: str, plugin_name: str,
-                     actions_list: list,
-                     triggers_list: list,
-                     indicator_receivers_list: list,
-                     alarm_receivers_list: list):
+                     modules_list_dict: dict):
     """
     #   生成入口文件
     参数说明：
     work_path:str,      #   当前工作区绝对路径
     plugin_name:str,    #   插件名称
-    actions_list:list,      #   动作列表
-    triggers_list:list,     #   触发器列表
-    indicator_receivers_list:list,      #   情报接收器列表
-    alarm_receivers_list:list,          #   告警接收器列表
+    modules_list_dict:dict,     #   组件的功能列表的字典
 
     生成失败时抛出异常
     """
@@ -911,12 +985,10 @@ def generateMainFile(work_path: str, plugin_name: str,
     try:
         main_data = {
             "pluginName": plugin_name.upper() + "_PLUGIN",
-            "actionClassees": actions_list,
-            "triggerClassees": triggers_list,
-            "indicatorReceiverClassees": indicator_receivers_list,
-            "alarmReceiverClassees": alarm_receivers_list
         }
 
+        main_data.update(modules_list_dict)
+
         file_path = os.path.join(work_path, "main.py")
 
         main_data = renderStrTemplate(main_data, main_template)
@@ -930,11 +1002,10 @@ def generateMainFile(work_path: str, plugin_name: str,
     log("info", r"入口文件文件 \main.py 生成完成")
 
 
-def generateHelpFile(work_path: str, yaml_data: dict):
+def generateHelpFile(yaml_data: dict):
     """
     #   生成帮助文件
     参数说明：
-    work_path:str,      #   当前工作区绝对路径
     yaml_data:dict,     #   yaml文件数据
 
     **此方法生成的文件基本用不上，预计在将来版本删除**
@@ -945,7 +1016,7 @@ def generateHelpFile(work_path: str, yaml_data: dict):
     log("info", r"生成帮助文件 \help.md 中")
 
     try:
-        file_path = os.path.join(work_path, "help.md")
+        file_path = os.path.join(os.getcwd(), "help.md")
         help_data = renderStrTemplate(yaml_data, help_template)
         with open(file_path, "w", encoding="utf-8", newline="\n") as file:
             file.write(help_data)
@@ -1109,7 +1180,7 @@ def argsSetupData(param_name: str, params: dict) -> list:
     构成参数中的关键数据：
     构成参数的类型（type）   #   在出现枚举集合时，可缺失，其他情况下不可缺失
     是否必填（required）  #   可缺失，缺省为False
-    默认值（default）    #   可缺失，缺省为None
+    默认值（default）    #   可缺失
     枚举集合（enum）   #   可缺失，缺省为None
     """
     params_list = []
@@ -1235,7 +1306,7 @@ def typeTransform(value_type: str, value_required: bool, value_default: None, va
 
 def renderStrTemplate(data: dict, template):
     """
-    #   根据template.py内的模板渲染字符串
+    #   根据templates.py内的模板渲染字符串
     参数说明：
     data,     #     需要渲染的数据
     template,    #      渲染的模板
@@ -1244,3 +1315,75 @@ def renderStrTemplate(data: dict, template):
     res = template.render(data)
 
     return res
+
+
+def loadIgnore():
+    """
+    #   读取忽略文件配置
+
+    没有传入数据或读取失败时将返回空列表
+    """
+    log("info", "读取配置文件中")
+    work_path = os.getcwd()
+    config_path = os.path.join(work_path, "config.ini")
+    try:
+        if os.path.exists(config_path):
+            config_data = configparser.ConfigParser()
+            config_data.read(config_path, encoding="utf-8")
+            ignore = config_data.get("ignore", "ignore", fallback="")
+            ignore = ignore.split(",")
+            return ignore
+        else:
+            log("attention", "未找到配置文件")
+            return []
+    except Exception as error:
+        log("warning", f"读取配置文件失败\n{error}")
+        return []
+
+
+def tarballOnlinePack(pack_name: str = ""):
+    """
+    参数说明：
+    pack_name:str,    #   包名称，不提供则根据插件定义文件自动生成
+
+    返回插件定义文件的数据
+    """
+
+    work_path = os.getcwd()
+
+    data = readGenerateFile(work_path, "plugin.spec.yaml")
+
+    #   获取插件定义文件中的元数据
+    vendor = data["vendor"]
+    name = data["name"]
+    version = data["version"]
+
+    #   在线包文件名
+    if pack_name:
+        tar_name = pack_name
+    else:
+        tar_name = f"{vendor}-{name}-{version}.tar.gz"
+
+    #   排除可能存在的离线包
+    offline_name = f"{vendor}-{name}-{version}-offline.tar.gz"
+
+    tar = tarfile.open(os.path.join(work_path, tar_name), "w:gz", format=tarfile.GNU_FORMAT)
+
+    #   默认排除项
+    ignore = ["__sdkcache__", "__pycache__", offline_name, f"{vendor}-{name}-{version}.tar.gz"]
+
+    ignore.extend(loadIgnore())
+
+    #   去重
+    ignore = list(set(ignore))
+
+    for file in os.listdir(work_path):
+        if file in ignore:
+            log("info", f"{file}已忽略")
+            continue
+        else:
+            tar.add(os.path.join(work_path, file), arcname=file)
+
+    tar.close()
+
+    return data
